
 XIncludeFile "Math.pbi"
 XIncludeFile "Vector2D.pbi"
 XIncludeFile "Line2D.pbi"



#PathFind_Reachable  = -1
#PathFind_Successful =  1
#PathFind_Calculate  =  0



Global PathFindMutex = CreateMutex()


; Knotenpunkt einer Wegsuche
Structure Node
 Position.Point2D
 Distance.f
 *ParentNode.Node
EndStructure

; Struktur einer Wegsuche
Structure PathTree
 Thread.l
 Start.Point2D
 Finish.Point2D
 CheckPaths.l
 Nodes.l
 Time.l
 *StartNode.Node
 *FinishNode.Node
 Index.l
 maxIndex.l
 *PathPoint.Point2D
EndStructure

; Mauer
Structure Wall Extends Line2D
 Detour.Point2D[2]
EndStructure


Structure SavePathLine Extends Line2D
 Temp$
EndStructure



; Liste fr Mauer
Global NewList Wall.Wall()

; Liste fr alle Knoten
Global NewList *SaveNode.Node()



; Lscht alle Mauern
Procedure ClearWall()
 ClearList(Wall())
EndProcedure

; Erstellt eine neue Mauer
Procedure AddWallXY(x1, y1, x2, y2)
 Protected Norm.Vector2D
 AddElement(Wall())
 With Wall()
  \Position\x = x1
  \Position\y = y1
  \Direction\x = x2-x1
  \Direction\y = y2-y1
  Vector2DNorm(\Direction, Norm)
  Vector2DSub(\Position, Norm, \Detour[0])
  Vector2DAdd(\Position, \Direction, \Detour[1])
  MoveVector2D(\Detour[1], Norm)
 EndWith
EndProcedure

; Erstellt eine neue Mauer
Procedure AddWallBox(x, y, Width, Height)
 AddWallXY(x, y, x+Width, y)
 AddWallXY(x, y, x, y+Height)
 AddWallXY(x+Width, y, x+Width, y+Height)
 AddWallXY(x, y+Height, x+Width, y+Height)
EndProcedure



; Berechnet die Distanz zwischen zwei Knoten
Procedure.f NodeDistance(*Node1.Node, *Node2.Node)
 Protected Distance.f = Distance2D(*Node1\Position, *Node2\Position)
 ProcedureReturn Distance
EndProcedure

; Erstellt einen Knoten
Procedure CreateNode(*ParentNode.Node, *Position.Point2D)
 Protected *Memory
 *Memory = AllocateMemory(SizeOf(Node))
 If *Memory
  AddElement(*SaveNode())
  *SaveNode() = *Memory
  *SaveNode()\ParentNode = *ParentNode
  If *Position
   CopyPoint2D(*Position, *SaveNode()\Position)
  Else
   SetPoint2D(*SaveNode()\Position, 0, 0)
  EndIf
  If *ParentNode
   *SaveNode()\Distance = *ParentNode\Distance + NodeDistance(*SaveNode(), *ParentNode)
  Else
   *SaveNode()\Distance = 0
  EndIf
 Else
;  ErrorHandler("@CreateNode()"+Chr(10)+"*Memory = 0")
 EndIf
 ProcedureReturn *Memory
EndProcedure


Procedure MinimizePath(*Node.Node)
 Protected PathLine.Line2D
 Protected *LastNode.Node = *Node, Collision = 0
 If *Node\ParentNode
  *Node = *Node\ParentNode 
 EndIf
 While *Node\ParentNode
  CopyPoint2D(*Node\ParentNode\Position, PathLine\Position) 
  Vector2DSub(*LastNode\Position, *Node\ParentNode\Position, PathLine\Direction)
  ForEach Wall() : With Wall()
   If CollisionLine2DLine2D(Wall(), PathLine, CollisionPoint.Point2D)
    Collision = #True
    Break
   EndIf
  EndWith : Next
  If Not Collision
   Debug "[i] Weg Optimiert"
   *LastNode\ParentNode = *Node\ParentNode
   *LastNode\Distance = *Node\ParentNode\Distance + NodeDistance(*LastNode, *Node\ParentNode)
  EndIf
  *Node = *Node\ParentNode 
 Wend
EndProcedure


Procedure UpdateDistance(*Node.Node)
 ForEach *SaveNode()
  If *SaveNode()\ParentNode = *Node
   *SaveNode()\Distance = *Node\Distance + NodeDistance(*SaveNode(), *Node)
   *Save = @*SaveNode()
   UpdateDistance(*SaveNode())
   ChangeCurrentElement(*SaveNode(), *Save)
  EndIf
 Next
EndProcedure


; HAUPTWEGSUCHE
Procedure CheckPath(*PathTree.PathTree, *Node.Node, *Point.Point2D, SavePathLine.Line2D(), Typ=0, Temp$="")
 
; Delay(1000)

 Protected PathLine.Line2D, PathLine2.Line2D, Value, SaveValue, Temp1.Vector2D, Temp2.Vector2D
 Protected *CollisionWall.Wall = 0, minDistance.f = 1e10
 Protected NewDistance.f, OldDistance.f, *NewNode.Node, *MyNode.Node = 0
  
 ; Weglinie wird erstellt
 CopyPoint2D(*Node\Position, PathLine\Position) 
 Vector2DSub(*Point, *Node\Position, PathLine\Direction)
 Debug Temp$+"[#] Line "+GetLine2D(PathLine)
 CopyPoint2D(*Point, PathLine\Position) 
 Vector2DSub(*Node\Position, *Point, PathLine\Direction)

 ; Kontrolle auf bereits untersuchte PathLines (auf diesem Weg)
 Debug Temp$+"[i] "+Str(CountList(SavePathLine()))+" SavePathLine"
 ForEach SavePathLine()
  If CompareLine2D(SavePathLine(), PathLine) 
   Debug Temp$+"[i] Abbruch (SavePathLine)"
   LastElement(SavePathLine())
   ProcedureReturn #False
  EndIf
 Next

 *PathTree\CheckPaths + 1

 ; Kollisionsberprfung : Weglinie mit Mauern
 ForEach Wall() : With Wall()
  Value = CollisionLine2DLine2D(Wall(), PathLine, CollisionPoint.Point2D)
  If Value
   Distance.f = Distance2D(PathLine\Position, CollisionPoint)
   If Distance < minDistance
    minDistance = Distance
    *CollisionWall = @Wall() 
    SaveValue = Value
   EndIf
  EndIf
 EndWith : Next

 AddElement(SavePathLine())
 CopyLine2D(PathLine, SavePathLine())
 AddElement(SavePathLine())
 CopyLine2D(PathLine2, SavePathLine())
 *SaveElement = @SavePathLine()

 ; Verhalten nach Prfung
 If *CollisionWall And SaveValue = 1

  ; Es gab eine Kollision und es werden 2 Umwege erstellt
  Debug Temp$+"[i] Kollision"
  CheckPath(*PathTree, *Node, *CollisionWall\Detour[0], SavePathLine(), 1, Temp$+"  ")
  ChangeCurrentElement(SavePathLine(), *SaveElement)
;  While NextElement(SavePathLine()) : DeleteElement(SavePathLine()) : Wend
  CheckPath(*PathTree, *Node, *CollisionWall\Detour[1], SavePathLine(), 1, Temp$+"  ")
  ChangeCurrentElement(SavePathLine(), *SaveElement)
;  While NextElement(SavePathLine()) : DeleteElement(SavePathLine()) : Wend
 
 ElseIf *CollisionWall

  ; Es gab eine Kollision und es werden 2 Umwege erstellt
  Debug Temp$+"[i] Kollision"
  Vector2DSub(*CollisionWall\Detour[0], *CollisionWall\Position, Temp1)  
  RotateVector2D(Temp1, #PI/2)
  MoveVector2D(Temp1, *CollisionWall\Position)  
  Vector2DSub(*CollisionWall\Detour[1], *CollisionWall\Position, Temp2)  
  Vector2DSub(Temp2, *CollisionWall\Direction, Temp2)  
  RotateVector2D(Temp2, #PI/2)
  MoveVector2D(Temp2, *CollisionWall\Position)  
  MoveVector2D(Temp2, *CollisionWall\Direction)  
  CheckPath(*PathTree, *Node, Temp1, SavePathLine(), 1, Temp$+"  ")
  ChangeCurrentElement(SavePathLine(), *SaveElement)
;  While NextElement(SavePathLine()) : DeleteElement(SavePathLine()) : Wend
  CheckPath(*PathTree, *Node, Temp2, SavePathLine(), 1, Temp$+"  ")
  ChangeCurrentElement(SavePathLine(), *SaveElement)
;  While NextElement(SavePathLine()) : DeleteElement(SavePathLine()) : Wend
 
 ElseIf Typ = 1
 
  ; Der Weg ist frei und es wird ein neuer Knoten erstellt
  Debug Temp$+"[i] Frei"
  
  ; Kontrolle auf bereits Bestehender Knoten (global)
  ForEach *SaveNode()
   If ComparePoint2D(*SaveNode()\Position, *Point)
    *MyNode = *SaveNode()
    Break
   EndIf
  Next
  
  ; Erstellen eines neuen Unterknotens
  If *MyNode
   Debug Temp$+"[*] Knoten vorhanden"
   NewDistance = *Node\Distance + NodeDistance(*Node, *MyNode)
   OldDistance = *MyNode\Distance
   Debug Temp$+"[?] New "+StrF(NewDistance)+"   Old "+StrF(OldDistance)
   If NewDistance < OldDistance
    *MyNode\ParentNode = *Node
    *MyNode\Distance = NewDistance
   EndIf
   MinimizePath(*MyNode)
   UpdateDistance(*MyNode)
  Else
   Debug Temp$+"[+] neuer Knoten"
   *NewNode = CreateNode(*Node, *Point)  
   MinimizePath(*NewNode)
   CheckPath(*PathTree, *NewNode, *PathTree\Finish, SavePathLine(), 0, Temp$+"  ")  
   ChangeCurrentElement(SavePathLine(), *SaveElement)
;   While NextElement(SavePathLine()) : DeleteElement(SavePathLine()) : Wend
  EndIf
  
 Else
  
  ; Der Weg ist frei und es wird der Zielknoten verndert
  Debug Temp$+"[i] Frei"

  If *PathTree\FinishNode\ParentNode
   Debug Temp$+"[*] Ziel vorhanden"
   NewDistance = *Node\Distance + NodeDistance(*Node, *PathTree\FinishNode)
   OldDistance = *PathTree\FinishNode\Distance
   Debug Temp$+"[?] New "+StrF(NewDistance)+"   Old "+StrF(OldDistance)
   If NewDistance < OldDistance
    *PathTree\FinishNode\ParentNode = *Node
    *PathTree\FinishNode\Distance = NewDistance
   EndIf
   MinimizePath(*PathTree\FinishNode)
  Else
   Debug Temp$+"[+] Ziel erstellt"
   NewDistance = *Node\Distance + NodeDistance(*Node, *PathTree\FinishNode)
   Debug Temp$+"[?] New "+StrF(NewDistance)
   *PathTree\FinishNode\ParentNode = *Node
   *PathTree\FinishNode\Distance = NewDistance
   MinimizePath(*PathTree\FinishNode)
  EndIf
   
 EndIf
 
 ProcedureReturn #True
 
EndProcedure


Procedure FreeTempPathFindMemory(*PathTree.PathTree)
 ForEach *SaveNode()
  FreeMemory(*SaveNode())
 Next
 ClearList(*SaveNode())
EndProcedure


Procedure FindPathThread(*PathTree.PathTree)
 Protected *ID.Node, Time, *Node.Node, n, Collision
 Protected CollisionPoint.Point2D, Optimiert, Point1.Point2D, Point2.Point2D, PathLine.Line2D
 NewList PathPoint.Point2D()
 LockMutex (PathFindMutex)
  ClearList(*SaveNode())
  Time = ElapsedMilliseconds()
  ; WegBaum erstellen
  Debug "----------"
  NewList SavePathLine.Line2D()
  ClearList(SavePathLine())
  CheckPath(*PathTree, *PathTree\StartNode, *PathTree\Finish, SavePathLine())
  Debug "----------"
  ; Rckverfolgung zum Start
  *Node = *PathTree\FinishNode
  If *Node\ParentNode
   While *Node\ParentNode
    InsertElement(PathPoint())
    CopyPoint2D(*Node\Position, PathPoint())
    *Node = *Node\ParentNode
   Wend
   ; Umwandelung zum WegPunkt-Array 
   *PathTree\maxIndex = CountList(PathPoint())
   *PathTree\PathPoint = AllocateMemory(*PathTree\maxIndex*8)
   ForEach PathPoint()
    n = ListIndex(PathPoint())
    PokeF(*PathTree\PathPoint+n*8  , PathPoint()\x)
    PokeF(*PathTree\PathPoint+n*8+4, PathPoint()\y)
   Next
   *PathTree\Index = -1
  Else
   *PathTree\PathPoint = 0
  EndIf
  *PathTree\Nodes = CountList(*SaveNode())
  FreeTempPathFindMemory(*PathTree)
  *PathTree\Time = ElapsedMilliseconds() - Time
 UnlockMutex (PathFindMutex)
EndProcedure


Procedure FindPath(*Start.Point2D, *Finish.Point2D)
 *PathTree.PathTree = AllocateMemory(SizeOf(PathTree))
 If *PathTree
  With *PathTree
   CopyPoint2D(*Start, \Start)
   CopyPoint2D(*Finish, \Finish)
   \StartNode  = CreateNode(0, \Start)
   \FinishNode = CreateNode(0, \Finish)
   \Thread     = CreateThread(@FindPathThread(), *PathTree)
  EndWith
  If *PathTree\Thread
   ProcedureReturn *PathTree
  EndIf
 EndIf 
EndProcedure

Procedure FreePath(*PathTree.PathTree)
 If IsThread(*PathTree\Thread)
  KillThread(*PathTree\Thread)
 EndIf
 If *PathTree\PathPoint
  FreeMemory(*PathTree\PathPoint)
 EndIf
 FreeMemory(*PathTree)
EndProcedure

Procedure ExaminePath(*PathTree.PathTree)
 If *PathTree And Not IsThread(*PathTree\Thread)
  *PathTree\Index = -1
  If *PathTree\PathPoint
   ProcedureReturn #PathFind_Successful
  Else
   ProcedureReturn #PathFind_Reachable
  EndIf
 Else
  ProcedureReturn #PathFind_Calculate 
 EndIf
EndProcedure

Procedure GetCheckPaths(*PathTree.PathTree)
 ProcedureReturn *PathTree\CheckPaths
EndProcedure

Procedure GetNodes(*PathTree.PathTree)
 ProcedureReturn *PathTree\Nodes
EndProcedure

Procedure GetPathFindTime(*PathTree.PathTree)
 ProcedureReturn *PathTree\Time
EndProcedure

Procedure NextPathPoint(*PathTree.PathTree)
 *PathTree\Index + 1
 If *PathTree\Index < *PathTree\maxIndex
  ProcedureReturn #True
 EndIf
EndProcedure

Procedure GetPathPoint(*PathTree.PathTree)
 ProcedureReturn *PathTree\PathPoint + *PathTree\Index*8
EndProcedure

; IDE Options = PureBasic 4.20 (Windows - x86)
; CursorPosition = 221
; FirstLine = 206
; Folding = ------
; EnableThread
; EnableOnError
; Executable = PathFind3.exe
; DisableDebugger