
;|
;|  Titel....: 2-Dimensionale Vektoren
;|  Datei....: Vector2D.pbi
;|  Datum....: 02.09.2008
;|  Inhalt...: * Macros und Proceduren fr 2-Dimensionale Vektorberechungen
;|________________________________________________________________________________________________




;  Vorrausgesetzte Includes
;
 
 XIncludeFile "Math.pbi"




;  Konstanten
;




;  Strukturen
;

 ; Structure eines 2D-Vektors
 Structure Vector2D
  x.f
  y.f
 EndStructure

 ; Structure eines 2D-Punkts
 Structure Point2D 
  x.f
  y.f
 EndStructure




;  Arrays und LinkedLists
;




;  Proceduren und Macros
;


 ; Setzt die Werte (x,y) in den Vektor ein 
 Macro SetVector2D(Vector2D, xValue, yValue)
  Vector2D\x = xValue
  Vector2D\y = yValue
 EndMacro
 
 ; Setzt die Werte (x,y) in den Punkt ein 
 Macro SetPoint2D(Point2D, xValue, yValue)
  Point2D\x = xValue
  Point2D\y = yValue
 EndMacro
 
 ; Gibt den Vektor als String aus
 Macro GetVector2D(Vector2D)
  "( "+StrF(Vector2D\x)+" | "+StrF(Vector2D\y)+" )"
 EndMacro
 
 ; Gibt den Punkt als String aus
 Macro GetPoint2D(Point2D)
  "( "+StrF(Point2D\x)+" | "+StrF(Point2D\y)+" )"
 EndMacro
 
 
 
 ; Umrechnung von Vektor zu Winkel
 Macro Vector2DToAngle(Vector2D, Angle)
  Angle = ATan(Vector2D\y/Vector2D\x) 
  If Vector2D\x < 0 : Angle + #PI : EndIf 
 EndMacro
 
 ; Umrechnung von Winkel zu normiertem Vektor
 Macro AngleToVector2D(Angle, Vector2D)
  Vector2D\x = Cos(Angle)
  Vector2D\y = Sin(Angle)
 EndMacro
 
 
 
 ; Gibt die Lnge des Vektors zurck
 Macro Vector2DLength(Vector2D)
  Sqr( Qua(Vector2D\x)+Qua(Vector2D\y) )
 EndMacro
 
 ; Gibt die quadratische Lnge des Vektors zurck
 Macro Vector2DQuaLength(Vector2D)
  ( Qua(Vector2D\x)+Qua(Vector2D\y) )
 EndMacro
 
 
 
 ; Berechnet den normierten Vektor zu einem Vektor
 Procedure Vector2DNorm(*Vector2D.Vector2D, *NormVector2D.Vector2D)
  Protected Length.f = Vector2DLength(*Vector2D)
  If Length
   *NormVector2D\x = *Vector2D\x / Length
   *NormVector2D\y = *Vector2D\y / Length
  Else
   *NormVector2D\x = 0
   *NormVector2D\y = 0
  EndIf
 EndProcedure
 
 ; Gibt das Skalarprodukt zweier Vektoren zurck
 Macro Vector2DScalar(Vector2D_1, Vector2D_2)
  ( Vector2D_1\x * Vector2D_2\x + Vector2D_1\y * Vector2D_2\y )
 EndMacro
 
 ; Gibt das Kreuzprodukt zweier Vektoren zurck
 Macro Vector2DCross(Vector2D_1, Vector2D_2)
  ( Vector2D_1\x * Vector2D_2\y - Vector2D_1\y * Vector2D_2\x )
 EndMacro
 
 ; Gibt den Winkel zwischen zwei Vektoren zurck
 Macro Vector2DAngle(Vector2D_1, Vector2D_2)
  ACos( Vector2DScalar(Vector2D_1, Vector2D_2) / (Vector2DLength(Vector2D_1)*Vector2DLength(Vector2D_2)) )
 EndMacro
 
 
 
 ; Berechnet die Summe zweier Vektor
 Macro Vector2DAdd(Vector2D_1, Vector2D_2, ResultVector2D)
  ResultVector2D\x = Vector2D_1\x + Vector2D_2\x
  ResultVector2D\y = Vector2D_1\y + Vector2D_2\y
 EndMacro
 
 ; Berechnet die Differenz zweier Vektor
 Macro Vector2DSub(Vector2D_1, Vector2D_2, ResultVector2D)
  ResultVector2D\x = Vector2D_1\x - Vector2D_2\x
  ResultVector2D\y = Vector2D_1\y - Vector2D_2\y
 EndMacro
 
 ; Multipliziert ein Vektor mit einem Faktor
 Macro Vector2DMul(Vector2D, Factor, ResultVector2D)
  ResultVector2D\x = Factor * Vector2D\x
  ResultVector2D\y = Factor * Vector2D\y
 EndMacro
 
 ; Rotiert einen Vektor (relativ)
 Macro Vector2DRotate(Vector2D, Angle, ResultVector2D)
  ResultVector2D\x = Cos(Angle)*Vector2D\x - Sin(Angle)*Vector2D\y
  ResultVector2D\y = Sin(Angle)*Vector2D\x + Cos(Angle)*Vector2D\y
 EndMacro 
 
 
 
 ; Addiert zu einem Vektor einen anderen hinzu
 Macro MoveVector2D(Vector2D, MoveVector2D)
  Vector2D\x + MoveVector2D\x
  Vector2D\y + MoveVector2D\y
 EndMacro
 
 ; ndert die Lnge des Vektor um einen Faktor
 Macro RaiseVector2D(Vector2D, Factor)
  Vector2D\x * Factor
  Vector2D\y * Factor
 EndMacro
 
 ; Normiert einen Vektor
 Procedure NormVector2D(*Vector2D.Vector2D)
  Protected Length.f = Vector2DLength(*Vector2D)
  If Length
   *Vector2D\x / Length
   *Vector2D\y / Length
  Else
   *Vector2D\x = 0
   *Vector2D\y = 0
  EndIf
 EndProcedure
 
 ; Rotiert einen Vektor (relativ)
 Procedure RotateVector2D(*Vector2D.Vector2D, Angle.f)
  Protected x.f, y.f
  x = *Vector2D\x : y = *Vector2D\y
  *Vector2D\x = Cos(Angle)*x - Sin(Angle)*y
  *Vector2D\y = Sin(Angle)*x + Cos(Angle)*y
 EndProcedure
 
 
 
 ; Kopiert einen Vektor
 Macro CopyVector2D(SourceVector2D, NewVector2D)
  NewVector2D\x = SourceVector2D\x
  NewVector2D\y = SourceVector2D\y
 EndMacro
 
 ; Kopiert einen Punkt
 Macro CopyPoint2D(SourcePoint2D, NewPoint2D)
  NewPoint2D\x = SourcePoint2D\x
  NewPoint2D\y = SourcePoint2D\y
 EndMacro
 
 ; Vergleicht zwei Vektoren auf gleichheit
 Procedure CompareVector2D(*Vector2D_1.Vector2D, *Vector2D_2.Vector2D)
  If *Vector2D_1\x = *Vector2D_2\x And *Vector2D_1\y = *Vector2D_2\y
   ProcedureReturn #True
  EndIf
 EndProcedure

 ; Vergleicht zwei Punkte auf gleichheit
 Procedure ComparePoint2D(*Point2D_1.Vector2D, *Point2D_2.Vector2D)
  If *Point2D_1\x = *Point2D_2\x And *Point2D_1\y = *Point2D_2\y
   ProcedureReturn #True
  EndIf
 EndProcedure  
 
 ; Tauscht zwei Vektoren miteinander
 Macro SwapVector2D(Vector2D_1, Vector2D_2)
  Swap Vector2D_1\x , Vector2D_2\x
  Swap Vector2D_1\y , Vector2D_2\y
 EndMacro

 ; Tauscht zwei Punkte miteinander
 Macro SwapPoint2D(Point2D_1, Point2D_2)
  Swap Point2D_1\x , Point2D_2\x
  Swap Point2D_1\y , Point2D_2\y
 EndMacro


   
 ; Gibt den Abstand zwischen zwei Punkte zurck
 Macro Distance2D(Point2D_1, Point2D_2)
  Sqr( Qua(Point2D_1\x-Point2D_2\x)+Qua(Point2D_1\y-Point2D_2\y) )
 EndMacro
 
 ; Gibt den quadratischen Abstand zwischen zwei Punkte zurck
 Macro QuaDistance2D(Point2D_1, Point2D_2)
  ( Qua(Point2D_1\x-Point2D_2\x)+Qua(Point2D_1\y-Point2D_2\y) )
 EndMacro
 
 
 
 ; Stellt einen Vektor an Position (x,y) dar
 Procedure DrawVector2D(*Vector2D.Vector2D, x, y, Color=#PB_Default)
  If Color > #PB_Default
   Line(x, y, *Vector2D\x, *Vector2D\y, Color)
  Else
   Line(x, y, *Vector2D\x, *Vector2D\y)
  EndIf
 EndProcedure
 
 ; Stellt einen Punkte mit dem Radius (r) dar
 Procedure DrawPoint2D(*Point2D.Point2D, r=0, Color=#PB_Default)
  If r
   If Color > #PB_Default
    Circle(*Point2D\x, *Point2D\y, r, Color)
   Else
    Circle(*Point2D\x, *Point2D\y, r)
   EndIf
  Else
   If Color > #PB_Default
    Plot(*Point2D\x, *Point2D\y, Color)
   Else
    Plot(*Point2D\x, *Point2D\y)
   EndIf   
  EndIf
 EndProcedure
; IDE Options = PureBasic 4.20 (Windows - x86)
; CursorPosition = 254
; FirstLine = 227
; Folding = ------